@@ -1,7 +1,7 @@
 package Farabi;
 use Mojo::Base 'Mojolicious';
 
-our $VERSION = '0.31';
+our $VERSION = '0.32';
 
 sub startup {
 	my $app = shift;
@@ -24,33 +24,15 @@ sub startup {
 	my $route = $app->routes;
 	$route->get('/')->to('editor#default');
 	$route->post('/')->to('editor#default');
-	$route->post('/help_search')->to('editor#help_search');
-	$route->post('/perl-tidy')->to('editor#perl_tidy');
-	$route->post('/perl-critic')->to('editor#perl_critic');
-	$route->post('/pod2html')->to('editor#pod2html');
-	$route->post('/pod-check')->to('editor#pod_check');
-	$route->post('/open-file')->to('editor#open_file');
-	$route->post('/find-file')->to('editor#find_file');
-	$route->post('/save-file')->to('editor#save_file');
-	$route->post('/open-url')->to('editor#open_url');
-	$route->post('/find-action')->to('editor#find_action');
-	$route->post('/run-perl')->to('editor#run_perl');
-	$route->post('/run-rakudo')->to('editor#run_rakudo');
-	$route->post('/run-niecza')->to('editor#run_niecza');
-	$route->post('/run-parrot')->to('editor#run_parrot');
-	$route->post('/find-duplicate-perl-code')
-	  ->to('editor#find_duplicate_perl_code');
-	$route->post('/dump-ppi-tree')->to('editor#dump_ppi_tree');
-	$route->post('/find-plugins')->to('editor#find_plugins');
-	$route->post('/repl-eval')->to('editor#repl_eval');
 
 	# Setup the Farabi database
-	eval {
-		$app->_setup_database;
-	};
-	if($@) {
+	eval { $app->_setup_database; };
+	if ($@) {
 		warn "Database not setup, reason: $@";
 	}
+
+	# Setup websocket message handler
+	$route->websocket('/websocket')->to('editor#websocket');
 }
 
 # Setup the Farabi database
@@ -74,12 +56,6 @@ SQL
 	$db->disconnect;
 }
 
-sub unsafe_features {
-
-	# Enable unsafe features by default for now
-	return 1;    # defined $ENV{FARABI_UNSAFE};
-}
-
 1;
 __END__
 
