@@ -21,6 +21,64 @@ $(function() {
 		indentWithTabs: true,
 		readOnly: true
 	});
+	
+	
+	var messages = [];
+
+	window.sendMessage = function(action, params, callback) {
+		var message = {
+			action   : action,
+			params   : params,
+			callback : callback,
+		};
+		console.info('sendMessage(' + action + ", ...)");
+		messages.push(message);
+		ws.send(JSON.stringify(message));
+	};
+
+	//--- Websockets support
+	if("WebSocket" in window) {
+		// WebSockets is supported
+		console.info("websockets supported");
+
+		window.ws = new WebSocket('ws://' + window.location.host + '/websocket');
+
+		ws.onopen = function () { 
+			console.info("WebSocket connection opened");
+
+			// Add an empty one editor tab on startup
+			AddEditorTab();
+		};
+
+		ws.onmessage = function(e) {
+			var message = messages.pop();
+			if(message.callback) {
+				var data = JSON.parse(e.data);
+				message.callback(data);
+			}
+		};
+		ws.onclose = function(e) {
+			console.info("WebSocket connection closed");
+		}
+
+		// Bug workaround: Firefox escape key kills the websocket connection...
+		if(navigator.userAgent.indexOf("Firefox") != -1) {
+			$(window).keydown(function(e) {
+				// Check for escape key
+				if (e.which == 27) {
+					// The following seems to fix the symptom but only in case the document has the focus
+					e.preventDefault();
+				}
+			});	
+		}
+
+	} else {
+		// No WebSockets support
+		alert("WebSockets is not supported here!\nBrowser: " + navigator.userAgent);
+	}
+	//--- Websockets support
+
+	
 
 	function displayHelp(cm) {
 		var selection = cm.getSelection();
@@ -233,7 +291,7 @@ $(function() {
 	});
 	
 	function _displayHelp(topic, bShowDialog) {
-		$.post('/help_search', {"topic": topic}, function(results) {
+		window.sendMessage('help_search', {"topic": topic}, function(results) {
 			if(results.length > 0) {
 				$(".topic").val(topic);
 				var html = '';
@@ -294,7 +352,7 @@ $(function() {
 	}
 
 	var podChanged = function(cm) {
-		$.post('/pod2html', {"source": cm.getValue()}, function(html) {
+		window.sendMessage('pod2html', {"source": cm.getValue()}, function(html) {
 			$('#pod-tab').html(html);
 		});
 	};
@@ -322,44 +380,29 @@ $(function() {
 	$(document).on('action-perl-tidy', function() {
 		var editor = GetCurrentEditor();
 		var outputEditor = GetOutputEditor();
-		$.post('/perl-tidy', {"source": editor.getValue()}, function(data) {
+		var source = editor.getValue();
+		window.sendMessage('perl-tidy', {"source": source}, function(data) {
 			if(data.error == '') {
 
-				// Save current cursor
-				var cursor = editor.getCursor();
-				var line_number = cursor.line;
-				var column_number = cursor.ch;
+				if(source != data.source) {
 
-				// Set tidied text to editor
-				editor.setValue(data.source);
+					// Save current cursor position
+					var cursor = editor.getCursor();
 
-				// Restore cursor
-				editor.setCursor({
-					line: line_number,
-					ch:   column_number,
-				});
+					// Set tidied text to editor
+					editor.setValue(data.source);
 
+					// Restore old cursor position
+					editor.setCursor(cursor);
+				}
+
+s
 			} else {
 				outputEditor.setValue('Error:\n' + data.error);
 			}
 		});
 	});
 
-	$(document).on('action-open-url', function() {
-		var url = prompt("Please Enter a http/https file location:" + 
-			"\ne.g https://raw.github.com/ihrd/uri/master/lib/URI.pm");
-		if(!url) {
-			alert("No URL was provided! Aborting");
-			return;
-		}
-		$.post('/open-url',
-			{ "url": url },
-			function(code) {
-				editor.setValue(code);
-			}
-		);
-	});
-
 	$(document).on('action-syntax-check', function() {
 		alert("TODO syntax check");
 	});
@@ -368,7 +411,7 @@ $(function() {
 
 	var podCheck = function() {
 		var editor = GetCurrentEditor();
-		$.post('/pod-check', {"source": editor.getValue()}, function(problems) {
+		window.sendMessage('pod-check', {"source": editor.getValue()}, function(problems) {
 			for (var i = 0; i < perlCheckerWidgets.length; i++) {
 				editor.removeLineWidget(perlCheckerWidgets[i].widget);
 			}
@@ -430,15 +473,8 @@ $(function() {
 
 	$(".topic").typeahead({
 		source : function(query, process) {
-			$.ajax({
-				type: 'POST',
-				url: '/typeahead',
-				async: false,
-				data: {'query': query},
-				success: function(matches) {
-					process(matches);
-				},
-				dataType: 'json',
+			window.sendMessage('/typeahead', {'query': query}, function(matches) {
+				process(matches);
 			});
 		}
 	}).change(function() {
@@ -506,7 +542,7 @@ $(function() {
 	$(document).on('action-save-file', function() {
 		var editor = GetCurrentEditor();
 		var filename = GetCurrentFilename();
-		$.post('/save-file', {"filename": filename, "source": editor.getValue()}, function(data) {
+		window.sendMessage('save-file', {"filename": filename, "source": editor.getValue()}, function(data) {
 			if(data.err == '') {
 				editor.markClean();
 				showStarAfterNameIfModified(editor);
@@ -530,7 +566,8 @@ $(function() {
 			source = editor.getValue();
 		}
 
-		$.post('/dump-ppi-tree', {"source": source}, function(data) {
+		sendMessage('dump-ppi-tree', {"source": source}, function(data) {
+
 			if(data.error == '') {
 				outputEditor.setValue(data.output);
 			} else {
@@ -544,7 +581,4 @@ $(function() {
 		});
 	});
 
-	// Add an empty one editor tab on startup
-	AddEditorTab();
-
 });
\ No newline at end of file
