@@ -22,6 +22,7 @@ $(function() {
 						'<tr>' +
 						'<th>Plugin</th>' +
 						'<th>Plugin Name</th>' +
+						'<th>Status</th>' +
 						'</tr>' +
 						'</thead>';
 					html += '<tbody>';
@@ -30,6 +31,7 @@ $(function() {
 						html += '<tr>' +
 							'<td>' +  plugin.id + '</td>' +
 							'<td>' +  plugin.name + '</td>' +
+							'<td>' +  plugin.status + '</td>' +
 							'</tr>';
 					}
 					html += '</tbody>';
